import logging
import os
from optparse import make_option

import polib
from django.conf import settings
from django.core.management.base import BaseCommand
# from deep_translator import GoogleTranslator
from django_deep_translator.services import GoogleTranslatorService
from django_deep_translator.utils import get_translator


logger = logging.getLogger(__name__)

default_options = () if not hasattr(BaseCommand, 'option_list') \
    else BaseCommand.option_list


class Command(BaseCommand):
    help = ('autotranslate all the message files that have been generated '
            'using the `makemessages` command.')

    option_list = default_options + (
        make_option('--locale', '-l', default=[], dest='locale', action='append',
                    help='autotranslate the message files for the given locale(s) (e.g. pt_BR). '
                         'can be used multiple times.'),
        make_option('--untranslated', '-u', default=False, dest='skip_translated', action='store_true',
                    help='autotranslate the fuzzy and empty messages only.'),
        make_option('--set-fuzzy', '-f', default=False, dest='set_fuzzy', action='store_true',
                    help='set the fuzzy flag on autotranslated messages.'),
        make_option('--source-language', '-s', default='en', dest='source_language', action='store',
                    help='override the default source language (en) used for translation.'),
    )

    def add_arguments(self, parser):
        # Previously, only the standard optparse library was supported and
        # you would have to extend the command option_list variable with optparse.make_option().
        # See: https://docs.djangoproject.com/en/1.8/howto/custom-management-commands/#accepting-optional-arguments
        # In django 1.8, these custom options can be added in the add_arguments()
        parser.add_argument('--locale', '-l', default=[], dest='locale', action='append',
                            help='autotranslate the message files for the given locale(s) (e.g. pt_BR). '
                                 'can be used multiple times.')
        parser.add_argument('--untranslated', '-u', default=False, dest='skip_translated', action='store_true',
                            help='autotranslate the fuzzy and empty messages only.')
        parser.add_argument('--set-fuzzy', '-f', default=False, dest='set_fuzzy', action='store_true',
                            help='set the fuzzy flag on autotranslated messages.')
        parser.add_argument('--source-language', '-s', default='en', dest='source_language', action='store',
                            help='override the default source language (en) used for translation.')

    def set_options(self, **options):
        self.locale = options['locale']
        self.skip_translated = options['skip_translated']
        self.set_fuzzy = options['set_fuzzy']
        self.source_language = options['source_language']

    def handle(self, *args, **options):
        self.set_options(**options)

        assert getattr(settings, 'USE_I18N', False), 'i18n framework is disabled'
        assert getattr(settings, 'LOCALE_PATHS', []), 'locale paths is not configured properly'
        for directory in settings.LOCALE_PATHS:
            # walk through all the paths
            # and find all the pot files
            for root, dirs, files in os.walk(directory):
                for file in files:
                    if not file.endswith('.po'):
                        # process file only
                        # if its a pot file
                        continue

                    # get the target language from the parent folder name
                    target_language = os.path.basename(os.path.dirname(root))

                    if self.locale and target_language not in self.locale:
                        logger.info('skipping translation for locale `{}`'.format(target_language))
                        continue

                    self.translate_file(root, file, target_language)

    def translate_file(self, root, file_name, target_language):
        """
        convenience method for translating a pot file

        :param root:            the absolute path of folder where the file is present
        :param file_name:       name of the file to be translated (it should be a pot file)
        :param target_language: language in which the file needs to be translated
        """
        logger.info('filling up translations for locale `{}`'.format(target_language))

        po = polib.pofile(os.path.join(root, file_name))
        for entry in po:
            if not entry.translated():
                translation = get_translator()
                entry.msgstr = translation.translate_string(text=entry.msgid,source_language=self.source_language, target_language=target_language)
                if self.set_fuzzy:
                    entry.flags.append('fuzzy')

        po.save()




import six

from django.conf import settings
from deep_translator import (GoogleTranslator, 
                            MyMemoryTranslator,
                            DeeplTranslator,
                            QcriTranslator,
                            PonsTranslator,
                            YandexTranslator,
                            MicrosoftTranslator,
                            PapagoTranslator,
                            LibreTranslator)

class BaseTranslatorService:
    """
    Defines the base methods that should be implemented
    """

    def translate_string(self,text, target_language, source_language='auto'):
        """
        Returns a single translated string literal for the target language.
        """
        raise NotImplementedError('.translate_string() must be overridden.')
    

class GoogleTranslatorService(BaseTranslatorService):


    def translate_string(self, text, target_language, source_language='auto'):
        self.service = GoogleTranslator(source=source_language, target=target_language)
        
        assert isinstance(text, six.string_types), '`text` should a string literal'
        return self.service.translate(text)
    

class MyMemoryTranslatorService(BaseTranslatorService):

    def translate_string(self, text, target_language, source_language='en'):
        self.service = MyMemoryTranslator(source=source_language, target=target_language)
        
        assert isinstance(text, six.string_types), '`text` should a string literal'
        return self.service.translate(text)
    
class DeeplTranslatorService(BaseTranslatorService):

    # deep-translator uses free deepl api by default. If you have the pro version then simply set the use_free_api to false

    def __init__(self):

        self.developer_key = getattr(settings, 'DEEPL_TRANSLATE_KEY', None)
        assert self.developer_key, ('`DEEPL_TRANSLATE_KEY` is not configured, '
                                    'it is required by `DeeplTranslatorService. '
                                    'Visit https://www.deepl.com/en/docs-api/ for more information on how to generate your Deepl api key')

    def translate_string(self, text, target_language, source_language='en', use_free_api=True):
        self.service = DeeplTranslator(
            source=source_language, 
            target=target_language, 
            api_key=self.developer_key,
            use_free_api=getattr(settings, 'DEEPL_FREE_API', use_free_api)
        )
        
        assert isinstance(text, six.string_types), '`text` should a string literal'
        return self.service.translate(text)
    

class QcriTranslatorService(BaseTranslatorService):

    # In order to use the QcriTranslator translator, you need to generate a free api key. 

    def __init__(self):

        self.developer_key = getattr(settings, 'QCRI_TRANSLATE_KEY', None)
        assert self.developer_key, ('`QCRI_TRANSLATE_KEY` is not configured, '
                                    'it is required by `QcriTranslatorService. '
                                    'Visit https://mt.qcri.org/api/ for more information')

    def translate_string(self, text, target_language, source_language='en'):
        self.service = QcriTranslator(
            self.developer_key,
        )
        
        assert isinstance(text, six.string_types), '`text` should a string literal'
        return self.service.translate(source=source_language, 
            target=target_language, text=text, domain="news")
    

class PonsTranslatorService(BaseTranslatorService):

    def translate_string(self, text, target_language, source_language='en'):
        self.service = PonsTranslator(source=source_language, target=target_language)
        
        assert isinstance(text, six.string_types), '`text` should a string literal'
        return self.service.translate(text)
    
class YandexTranslatorService(BaseTranslatorService):

    # You need to require a private api key if you want to use the yandex translator      

    def __init__(self):

        self.developer_key = getattr(settings, 'YANDEX_TRANSLATE_KEY', None)
        assert self.developer_key, ('`YANDEX_TRANSLATE_KEY` is not configured, '
                                    'it is required by `YandexTranslatorService. '
                                    'You need to require a private api key if you want to use the yandex translator. Visit the official website for more information about how to get one')

    def translate_string(self, text, target_language, source_language='en'):
        self.service = YandexTranslator(
            self.developer_key,
        )
        
        assert isinstance(text, six.string_types), '`text` should a string literal'
        return self.service.translate(source=source_language, 
            target=target_language, text=text)
    
class MicrosoftTranslatorService(BaseTranslatorService):

    """
    You need to require an api key if you want to use the microsoft translator. 
    Visit the official website for more information about how to get one. Microsoft offers a free tier 0 subscription (2 million characters per month).
    """

    def __init__(self):

        self.developer_key = getattr(settings, 'MICROSOFT_TRANSLATE_KEY', None)
        assert self.developer_key, ('`MICROSOFT_TRANSLATE_KEY` is not configured, '
                                    'it is required by `MicrosoftTranslatorService. '
                                    'You need to require an api key if you want to use the microsoft translator. Visit the official website for more' 
                                    ' information about how to get one. Microsoft offers a free tier 0 subscription (2 million characters per month).')

    def translate_string(self, text, target_language):
        self.service = MicrosoftTranslator(
            target=target_language, 
            api_key=self.developer_key,
        )
        
        assert isinstance(text, six.string_types), '`text` should a string literal'
        return self.service.translate(text=text)
    

class PapagoTranslatorService(BaseTranslatorService):
    """
    You need to require an api key if you want to use the microsoft translator. 
    Visit the official website for more information about how to get one. Microsoft offers a free tier 0 subscription (2 million characters per month).
    """

    def __init__(self):

        self.client_id = getattr(settings, 'PAPAGO_CLIENT_ID', None)
        assert self.secret_key, ('`PAPAGO_CLIENT_ID` is not configured, '
                                    'it is required by `PapagoTranslatorService. '
                                    'You need to require a client id if you want to use the papago translator.'
                                    'Visit the official website for more information about how to get one.')

        self.secret_key = getattr(settings, 'PAPAGO_SECRET_KEY', None)
        assert self.secret_key, ('`PAPAGO_SECRET_KEY` is not configured, '
                                    'it is required by `PapagoTranslatorService. '
                                    'You need to require a client id if you want to use the papago translator.'
                                    'Visit the official website for more information about how to get one.')


    def translate_string(self, text, target_language, source_language="en"):
        self.service = PapagoTranslator(
            target=target_language, 
            source=source_language,
            client_id=self.client_id,
            secret_key=self.secret_key,
        )
        
        assert isinstance(text, six.string_types), '`text` should a string literal'
        return self.service.translate(text=text)
    

class LibreTranslatorService(BaseTranslatorService):
    """
    Libre translate has multiple mirrors which can be used for the API endpoint. 
    Some require an API key to be used. By default the base url is set to libretranslate.de . 
    This can be set using the “base_url” input parameter.
    """

    def translate_string(self, text, target_language, source_language="en", ):
        self.service = LibreTranslator(
            target=target_language, 
            source=source_language,
            base_url=getattr(settings, 'LIBRE_TRANSLATE_MIRROR_URL', 'https://libretranslate.com/'), 
            client_id=self.client_id,
            api_key=getattr(settings, 'LIBRE_TRANSLATE_KEY', None)
        )
        
        assert isinstance(text, six.string_types), '`text` should a string literal'
        return self.service.translate(text=text)
    






"""
The `compat` module provides support for backwards compatibility with older
versions of Django/Python, and compatibility wrappers around optional packages.
"""
try:
    # Available in Python 3.1+
    import importlib
except ImportError:
    # Will be removed in Django 1.9
    from django.utils import importlib